@@ -8,7 +8,7 @@ package IO::Async::OS::cygwin;
 use strict;
 use warnings;
 
-our $VERSION = '0.61';
+our $VERSION = '0.63';
 
 our @ISA = qw( IO::Async::OS::_Base );
 
@@ -16,6 +16,9 @@ our @ISA = qw( IO::Async::OS::_Base );
 # emulation of poll() isn't quite perfect. It needs POLLPRI
 use constant HAVE_POLL_CONNECT_POLLPRI => 1;
 
+# Also select() only reports connect() failures by evec, not wvec
+use constant HAVE_SELECT_CONNECT_EVEC => 1;
+
 =head1 NAME
 
 C<IO::Async::OS::cygwin> - operating system abstractions on C<cygwin> for C<IO::Async>
